/********************************************************************
	created:		2001/03/24
	file name:		D:\grafik\MPREstd\UAString.h
	OS specific:	Yes
	author:			John Juul Jensen

					Copyright 2001
					[COPYRIGHT NOTICE]
	
	contains:		CUAString
*********************************************************************/

#ifndef __UASTRING_H
#define __UASTRING_H


#include "wchar.h"

/********************************************************************
	class name:	CUAString
	created:	2001/03/24
	author:		John Juul Jensen
	
	purpose:	The CUAString class is a string class based on the 
				unicode standard. All text is kept in unicode 
				internally and this is also the preferred external use.
				For accurate conversion between ascii and unicode, a 
				codepage can be specified.
				The class implements the conversion function string->float.
				A few basic search functions, similar to the ones found 
				in std::string, has also been implemented.
				All string lengths in this this class are assumed to be 
				excluding the null character.
*********************************************************************/
class CUAString 
{
public: // Constructors / Destructor
	CUAString();
	CUAString( const CUAString& _in);
	CUAString( const wchar_t* _in);
	CUAString( const char* _in);
	CUAString( const double _in);
	~CUAString();

public: // Operators
	const CUAString& operator=( const CUAString& _in);
	const CUAString& operator=( const wchar_t* _in);
	const CUAString& operator=( const char* _in);
	const CUAString& operator=( const double _in);

	const CUAString& operator+=( const CUAString& _in);
	const CUAString& operator+=( const wchar_t* _in);
	const CUAString& operator+=( const char* _in);
	const CUAString& operator+=( const double _in);

	friend CUAString operator+( const CUAString& string1, const CUAString& string2);
	friend CUAString operator+( const CUAString& string1, const wchar_t* string2);
	friend CUAString operator+( const wchar_t* string1, const CUAString& string2);
	friend CUAString operator+( const CUAString& string1, const char* string2);
	friend CUAString operator+( const char* string1, const CUAString& string2);
	friend CUAString operator+( const CUAString& string1, const double string2);
	friend CUAString operator+( const double string1, const CUAString& string2);

	bool operator==( const CUAString& _in);
	bool operator==( const wchar_t* _in);
	bool operator!=( const CUAString& _in);
	bool operator!=( const wchar_t* _in);

public: // Search functions
	unsigned int FindFirstOf( unsigned int startPos, wchar_t* filter);
	unsigned int FindFirstNotOf( unsigned int startPos, wchar_t* filter);
	CUAString SubString( unsigned int startPos, int length = -1);

public: // Extract/Conversion routines
	double			AsDouble( bool extract = false);
	long			AsLong( bool extract = false);
	CUAString		AsToken( wchar_t startChar = L'[', wchar_t endChar = L']', bool extract = false);
	const wchar_t*	AsUnicode();
	const char*		AsAscii( int _codepage = 0);
	

public: // Misc functions
	int Codepage();
	bool Codepage( int _codepage);

	void MakeLower();
	void MakeUpper();

	bool Empty();

protected: // functions

	void BuildAscii();
	void Clear();

	void DeAllocateString();
	void AllocateString( unsigned int _size);
	void ReAllocateString( unsigned int _size);
	void AppendString( const wchar_t* _str, unsigned int _length);
	void AppendDouble( double num);

protected: // variables

	unsigned int codepage;

	unsigned int uStringLength; // length excluding terminating null character
	unsigned int uStringBufferSize; // size of buffer
	wchar_t* uString;

	unsigned int aStringLength;
	char* aString; // length excluding terminating null character
};


#endif